package ch.usi.inf.sape.tracer.analyzer.cdg;

import java.util.List;

import org.objectweb.asm.tree.AbstractInsnNode;

public class BasicBlock<V> implements Label<V> {
	
	private final ControlFlowGraph<V,?> cfg;
	private final V label;
	private final V start;
	private final V end;
	private final int id;
	
	
	public BasicBlock(final V start, final V end, final V label, final int id, final ControlFlowGraph<V,?> cfg){
		this.cfg = cfg;
		this.label = label;
		this.start = start;
		this.end = end;
		this.id = id;
	}

	/**
	 * We use the instruction type, because the index may change
	 * given the instrumentation and it is not reliable.
	 */
	@Override
	public V getLabel() {
		return label;
	}

	/**
	 * We use the instruction type, because the index may change
	 * given the instrumentation and it is not reliable.
	 * 
	 * @return the start
	 */
	public V getStart() {
		return start;
	}

	/**
	 * We use the instruction type, because the index may change
	 * given the instrumentation and it is not reliable.
	 * 
	 * @return the end
	 */
	public V getEnd() {
		return end;
	}

	@Override
	public int getID() {
		return this.id;
	}

	@Override
	public String toString() {
		InstructionList<V> insns = this.cfg.getInstructions();
		StringBuffer buffer = new StringBuffer();
		int startIndex = insns.indexOf(this.start);
		int endIndex = insns.indexOf(this.end);
		
		if (startIndex == -1 || endIndex == -1)
			return "NOP";
		
		for (int i = startIndex; i<= endIndex; i++){
			buffer.append(insns.get(i).getName()).append("\n");
		}
		return buffer.toString();
	}
}
