package ch.usi.inf.sape.tracer.analyzer.gui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ch.usi.inf.sape.tracer.analyzer.Binding;
import ch.usi.inf.sape.tracer.analyzer.TimeTravel;
import ch.usi.inf.sape.tracer.analyzer.locations.MemoryLocation;

public class Element {

	private final long time;
	
	private final Binding<? extends MemoryLocation>[] bindings;
	
	private static Map<TimeTravel, List<Element>> visited = new HashMap<>();
	
	private final Set<Binding<? extends MemoryLocation>> usedBindings = new HashSet<>();

	public Element(Binding<? extends MemoryLocation>[] bindings, long time){
		this.bindings = bindings;
		this.time = time;
	}
	
	public void addUsedBinding(Binding<? extends MemoryLocation> binding){
		this.usedBindings.add(binding);
	}
	
	public static Element findOrGenerateElement(TimeTravel tt, long travelToOrBefore){
		Element element;
		Binding<? extends MemoryLocation>[] bindings = tt.getElementsAtTime(travelToOrBefore);
		long preciseTime = findPreciseTime(bindings);
 
		List<Element> states = visited.get(tt);
		if (states == null){
			states = new ArrayList<>();
			states.add(new Element(bindings,preciseTime));
			visited.put(tt, states);
		}
		
		return findOrGenerateElement(bindings, states, preciseTime);
	}
	
	
	private static Element findOrGenerateElement(Binding<? extends MemoryLocation>[] bindings, List<Element> states, long preciseTime){
		Element element = null;
		for (Element state : states) {
			if (state.time ==  preciseTime){
				element = state; 
			}
		}
		
		if (element == null){
			element = new Element(bindings, preciseTime);
			states.add(element);
		}
		
		return element;
	}
	
	private static Element generateElement(TimeTravel tt, long time){
		Binding<? extends MemoryLocation>[] bindings = tt.getElementsAtTime(time);
		return new Element(bindings, findPreciseTime(bindings));
	}
	
	private static long findPreciseTime(Binding<? extends MemoryLocation>[] bindings){
		long max = -1;
		for (Binding<? extends MemoryLocation> binding : bindings) {
			if (binding.getProvider().getEventTime() > max)
				max = binding.getProvider().getEventTime();
		}
		
		return max;
	}

	public static String toStringAll() {
		StringBuffer buffer = new StringBuffer();
		for (TimeTravel tt: visited.keySet()){
			List<Element> elements = visited.get(tt);
			Collections.sort(elements, new Comparator<Element>() {

				@Override
				public int compare(Element o1, Element o2) {
					return o1.time > o2.time ? 1 : (o1.time == o2.time ? 0 : -1);
				}
			});
			
			for (Element element : elements) {
				buffer.append(element.toString()).append("\n");
			}
		}
		return buffer.toString();
	}
	
	@Override
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("[");
		for (int i = 0; i < bindings.length; i++) {
			boolean used = usedBindings.contains(bindings[i]);
			buffer.append(bindings[i].getValue().toString());
			if (i < bindings.length - 1)
				buffer.append(" , ");
		}
		buffer.append("]");
		return buffer.toString();
	}
	
	
}
