package ch.usi.inf.sape.tracer.test;

import java.util.List;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestWatcher;
import org.junit.runner.Description;
import static org.junit.Assert.*;


import ch.usi.inf.sape.tracer.analyzer.Binding;
import ch.usi.inf.sape.tracer.analyzer.Bytecodes;
import ch.usi.inf.sape.tracer.analyzer.QueryFacade;
import ch.usi.inf.sape.tracer.analyzer.bytecodeops.BytecodeEvent;
import ch.usi.inf.sape.tracer.analyzer.RuntimeEnv;
import ch.usi.inf.sape.tracer.analyzer.locations.MemoryLocation;
import ch.usi.inf.sape.tracer.analyzer.locations.MemoryLocation.LocationType;
import ch.usi.inf.sape.tracer.test.snippets.BytecodeBehaviorSnippetCollection;

/**
 * The tests in this classes are aimed at testing
 * the pure functionality of each bytecode.
 * 
 * @author reza
 *
 */
public class BytecodeBehaviorTest {

	/**
	 * Logs each test method's name upon execution
	 */
	@Rule
	public TestWatcher testWatcher = new TestWatcher() {
		@Override
		protected void starting(final Description description) {
			String methodName = description.getMethodName();
			System.out.println("Test Method: " + methodName);
		}
	};
	
	@Test
	public void iloadTest(){
		BytecodeBehaviorSnippetCollection.iloadTest();
		RuntimeEnv session = RuntimeEnv.getInstance();
		List<BytecodeEvent> ops = session.getAllBytecodeEvents();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfOpCode(1, Bytecodes.ILOAD, session);
		Binding<? extends MemoryLocation> defOperandStack = null;
		Binding<? extends MemoryLocation> usedLocalVar = null;
		
		for (Binding<? extends MemoryLocation> def :criterion.getDefs()){
			if (def.getMemoryLocation().isStackSlot()){
				defOperandStack = def;
				break;
			}
		}
		
		assertNotNull("An operand stack value has to be defined ", defOperandStack);
		
		Binding<? extends MemoryLocation>[] uses = criterion.getUsesForDef(defOperandStack);
		assertEquals("Only a local should be used .", uses.length, 1);
		usedLocalVar = uses[0];
		assertEquals("Local variable location must be used.", usedLocalVar.getMemoryLocation().getLocationType(), LocationType.LOCAL_VAR);
	}

}
