package ch.usi.inf.sape.tracer.test;

import static org.junit.Assert.assertEquals;

import java.util.List;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestName;
import org.junit.rules.TestWatcher;
import org.junit.runner.Description;

import ch.usi.inf.sape.tracer.analyzer.Analyzer;
import ch.usi.inf.sape.tracer.analyzer.EventConstants;
import ch.usi.inf.sape.tracer.analyzer.QueryFacade;
import ch.usi.inf.sape.tracer.analyzer.bytecodeops.BytecodeEvent;
import ch.usi.inf.sape.tracer.analyzer.RuntimeEnv;
import ch.usi.inf.sape.tracer.test.snippets.CodeSnippetCollection;

/**
 * TODO: add explicit test for a case where a basic
 * block was introduced because a jump instruction 
 * was targeting it.
 * 
 * TODO: Use ASM to generate the bytecode array, 
 * instead of writing code snippets in java.
 * 
 * TODO: Add naming convention note
 * 
 * 
 * TODO: Add tests with short circuited conditions
 * such as or or switch case with no break between
 * two cases containing code
 * 
 * TODO: One to one mapping between tests and snippets
 * 
 * TODO: Add tests for jump
 * TODO: Add tests for PEIs
 * TODO: Add tests for throw instruction
 * TODO: Add tests for invocations
 * TODO: Add tests for differentiating between PEIs
 * TODO: Add tests for nested and combined conditions
 * 
 * and sole control flow instructions
 * 
 * We specify the predicates for the slicing tests
 * based on the relative source line number of the 
 * results.
 *  
 * @author reza
 *
 */
public class ControlFlowTest {
	
	/**
	 * Logs each test method's name upon execution
	 */
	@Rule
	public TestWatcher testWatcher = new TestWatcher() {
		@Override
		protected void starting(final Description description) {
			String methodName = description.getMethodName();
			System.out.println("Test Method: " + methodName);
		}
	};
	
	/* ******************************************************************************************
	 *********ifop Includes bytecodes ifeq, ifne, iflt, ifge, ifgt, ifle, ifnull, ifnonnull******
	 ********************************************************************************************/
	
	@Test
	public void ifopWithoutPEITest1(){
		CodeSnippetCollection.ifopWithoutPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 5, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ifopWithoutPEITest2(){
		CodeSnippetCollection.ifopWithoutPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement within if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the inner if condition.", slicingCriterionLineNo - 2, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 3, secondController.getSourceCodeLineNumber());
	}

	/**
	 * Test the statement after if body when the inner if is not taken.
	 */
	@Test
	public void ifopWithoutPEITest3(){
		CodeSnippetCollection.ifopWithoutPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 5, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", -1, secondController.getSourceCodeLineNumber());
	}

	/**
	 * Test an invocation PEI;
	 */
	@Test
	public void ifopWithPEIOffTest1(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ifopWithPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		//don't include PEI
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 5, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());

	}

	/**
	 * Test a non-invocation PEI
	 */
	@Test
	public void ifopWithPEIOffTest2(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ifopWithPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 5, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	/**
	 * The case when the inner if condition is not taken
	 */
	@Test
	public void ifopWithPEIOffTest3(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ifopWithPEI3();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 5, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	/**
	 * Testing the case an Xreturn is used in the if block
	 */
	@Test
	public void ifopWithPEIOffTest4(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ifopWithPEI4();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the inner if condition.", slicingCriterionLineNo - 5, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 6, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ifopWithPEIOnTest1(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ifopWithPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		//including EPIs
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the invocation.", slicingCriterionLineNo - 3, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the inner if.", slicingCriterionLineNo - 4, secondController.getSourceCodeLineNumber());

	}

	@Test
	public void ifopWithPEIOnTest2(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ifopWithPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement within if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the PEI.", slicingCriterionLineNo - 3, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the inner if.", slicingCriterionLineNo - 4, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ifopWithPEIOnTest3(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ifopWithPEI3();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the inner if stmt.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if stmt.", slicingCriterionLineNo- 5, secondController.getSourceCodeLineNumber());
	}

	
	
	
	/* *********************************************************************************************************
	 *********if_icmpX Includes bytecodes IF_ICMPEQ, IF_ICMPNE, IF_ICMPLT, IF_ICMPGE, IF_ICMPGT, IF_ICMPLE******
	 ***********************************************************************************************************/

	@Test
	public void ificmpXopWithoutPEITest1(){
		CodeSnippetCollection.ificmpXopWithoutPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ificmpXopWithoutPEITest2(){
		CodeSnippetCollection.ificmpXopWithoutPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement within if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the inner if condition.", slicingCriterionLineNo - 1, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 2, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ificmpXopWithoutPEITest3(){
		CodeSnippetCollection.ificmpXopWithoutPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", -1, secondController.getSourceCodeLineNumber());
	}

	
	@Test
	public void ificmpXopWithPEIOffTest1(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ificmpXopWithPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		//don't include PEI
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());

	}

	@Test
	public void ificmpXopWithPEIOffTest2(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ificmpXopWithPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ificmpXopWithPEIOffTest3(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ificmpXopWithPEI3();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ificmpXopWithPEIOnTest1(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ificmpXopWithPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		//including EPIs
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the invocation.", slicingCriterionLineNo - 2, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the inner if.", slicingCriterionLineNo - 3, secondController.getSourceCodeLineNumber());

	}

	@Test
	public void ificmpXopWithPEIOnTest2(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ificmpXopWithPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the PEI.", slicingCriterionLineNo - 2, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the inner if.", slicingCriterionLineNo - 3, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ificmpXopWithPEIOnTest3(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ificmpXopWithPEI3();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the inner if stmt.", slicingCriterionLineNo - 3, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if stmt.", slicingCriterionLineNo- 4, secondController.getSourceCodeLineNumber());
	}

	
	
	/* ******************************************************************************************
	 *******************IF_ACMP_X Includes bytecodes IF_ACMP_EQ and IF_ACMP_NE*******************
	 ********************************************************************************************/
	
	@Test
	public void ifacmpXopWithoutPEITest1(){
		CodeSnippetCollection.ifacmpXopWithoutPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement after if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ifacmpXopWithoutPEITest2(){
		CodeSnippetCollection.ifacmpXopWithoutPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement within if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the inner if condition.", slicingCriterionLineNo - 1, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 2, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ifacmpXopWithoutPEITest3(){
		CodeSnippetCollection.ifacmpXopWithoutPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", -1, secondController.getSourceCodeLineNumber());
	}

	
	@Test
	public void ifacmpXopWithPEIOffTest1(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ifacmpXopWithPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		//don't include PEI
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());

	}

	@Test
	public void ifacmpXopWithPEIOffTest2(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ifacmpXopWithPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ifacmpXopWithPEIOffTest3(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.ifacmpXopWithPEI3();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the outer if condition.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ifacmpXopWithPEIOnTest1(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ifacmpXopWithPEI1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement after inner if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		//including EPIs
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the invocation.", slicingCriterionLineNo - 2, directController.getSourceCodeLineNumber());
		
		//Test the second controller of the statement  
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the inner if.", slicingCriterionLineNo - 3, secondController.getSourceCodeLineNumber());

	}

	@Test
	public void ifacmpXopWithPEIOnTest2(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ifacmpXopWithPEI2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the PEI.", slicingCriterionLineNo - 2, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the inner if.", slicingCriterionLineNo - 3, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void ifacmpXopWithPEIOnTest3(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.ifacmpXopWithPEI3();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the inner if stmt.", slicingCriterionLineNo - 3, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if stmt.", slicingCriterionLineNo- 4, secondController.getSourceCodeLineNumber());
	}

	
	
	/* ***************************************************************************************************
	 **********************************   tableswitch and lookupswitch   *********************************
	 *****************************************************************************************************/

	@Test
	public void tableSwitchWithoutPEITest1(){
		CodeSnippetCollection.tableSwitchWithoutPEI1();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement within a case block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the tableswitch.", slicingCriterionLineNo - 8, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 9, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void tableSwitchWithoutPEITest2(){
		CodeSnippetCollection.tableSwitchWithoutPEI1();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement after tableswitch block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 15, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", -1, secondController.getSourceCodeLineNumber());
	}
	
	@Test
	public void tableSwitchWithPEIOffTest1(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.tableSwitchWithPEI1();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement within default block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the tableswitch.", slicingCriterionLineNo - 11, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 12, secondController.getSourceCodeLineNumber());
	}

	@Test
	public void tableSwitchWithPEIOnTest1(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.tableSwitchWithPEI1();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement within default block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the tableswitch.", slicingCriterionLineNo - 11, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 12, secondController.getSourceCodeLineNumber());
	}

	
	@Test
	public void lookupSwitchWithImplicitlyAddedConditionTest1(){
		CodeSnippetCollection.lookupSwitchWithoutPEI1();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement within a case block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the implicit if defined by lookup.", slicingCriterionLineNo - 5, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the lookup.", slicingCriterionLineNo - 5, secondController.getSourceCodeLineNumber());
		
		BytecodeEvent thirdController = QueryFacade.findControllerForBytecode(secondController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 6, thirdController.getSourceCodeLineNumber());
	}
	
	/**
	 * TODO: refactor the test and move the invocation before the
	 * switch.
	 * Add a simple test for lookupswitch to include integer
	 */
	@Test
	public void lookupSwitchWithPEITest1(){
		CodeSnippetCollection.lookupSwitchWithPEI1();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement within default block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the tableswitch.", slicingCriterionLineNo - 11, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 12, secondController.getSourceCodeLineNumber());
	}

	@Test
	public void lookupSwitchWithPEIOffTest2(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.lookupSwitchWithPEI2();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement after switch statement
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 15, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", -1, secondController.getSourceCodeLineNumber());
	}

	@Test
	public void lookupSwitchWithPEIOnTest1(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.lookupSwitchWithPEI2();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement after switch statement
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the switch stmt.", slicingCriterionLineNo - 9, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 14, secondController.getSourceCodeLineNumber());
	}

	


	/* ***************************************************************************************************
	 ******Invocation includes bytecodes invokevirtual, invokestatic, invokespecial, invokeinterface******
	 *****************************************************************************************************/
	
	/**
	 * TODO: refactor
	 */
	@Test
	public void invocationStaticOpWithPEIOffTest1(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.invocationStatic();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement within if body
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the if statement.", slicingCriterionLineNo - 1, directController.getSourceCodeLineNumber());
	}
	
	/* ***********************************************************************************
	 * ***********************Tests for exception throwing instructions******************* 
	 *************************************************************************************/
	@Test
	public void runtimeExceptionWithPEIOffTest1(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.catchRuntimeExceptionAfterInvocation();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement within catch block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the exception throwing insn.", slicingCriterionLineNo - 3, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the if.", slicingCriterionLineNo - 4, secondController.getSourceCodeLineNumber());

	}
	
	@Test
	public void runtimeExceptionWithPEIOnTest1(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.catchRuntimeException();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		
		//Test the statement within catch block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the exception throwing insn.", slicingCriterionLineNo - 2, directController.getSourceCodeLineNumber());
	}

	@Test
	public void runtimeExceptionWithPEIOnTest2(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.catchRuntimeExceptionAfterInvocation();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement within catch block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the exception throwing insn.", slicingCriterionLineNo + 6, directController.getSourceCodeLineNumber());
	}

	@Test
	public void checkedExceptionWithPEIOffTest1(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.catchCheckedException();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement within catch block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		
		assertEquals("Must be control dependent on the exception throwing insn.", slicingCriterionLineNo + 7, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the if condition.", slicingCriterionLineNo+ 6, secondController.getSourceCodeLineNumber() );
	}
	
	@Test
	public void checkedExceptionWithPEIOnTest1(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.catchCheckedException();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		//Test the statement within catch block
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the exception throwing insn.", slicingCriterionLineNo + 7, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", slicingCriterionLineNo + 7, secondController.getSourceCodeLineNumber());
		
		BytecodeEvent thirdController = QueryFacade.findControllerForBytecode(secondController, session);
		assertEquals("Must be control dependent on the new bytecode.", slicingCriterionLineNo + 7, thirdController.getSourceCodeLineNumber());
		
		BytecodeEvent forthController = QueryFacade.findControllerForBytecode(thirdController, session);
		assertEquals("Must be control dependent on the if.", slicingCriterionLineNo + 6, forthController.getSourceCodeLineNumber());
	}
	

	/* ***********************************************************************************
	 * ***********************  Tests for Loops and goto bytecode  *********************** 
	 *************************************************************************************/

	/**
	 * Find controller for the statement after the 
	 * for loop
	 */
	@Test
	public void forLoopWithoutPEITest1(){
		CodeSnippetCollection.forLoopWithoutPEI();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
	}

	/**
	 * Find controller for the statement within 
	 * for loop
	 */
	@Test
	public void forLoopWithoutPEITest2(){
		CodeSnippetCollection.forLoopWithoutPEI2();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the loop condition.", slicingCriterionLineNo - 1, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the loop condition.", slicingCriterionLineNo - 1, secondController.getSourceCodeLineNumber());
		
		BytecodeEvent thirdController = QueryFacade.findControllerForBytecode(secondController, session);
		assertEquals("Must be control dependent on the loop condition.", slicingCriterionLineNo - 1, thirdController.getSourceCodeLineNumber());
		
		BytecodeEvent forthController = QueryFacade.findControllerForBytecode(thirdController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 2, forthController.getSourceCodeLineNumber());
	}
	
	/**
	 * Find controller for the statement after the 
	 * for loop
	 */
	@Test
	public void forLoopWithPEIOffTest1(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.forLoopWithPEI1();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 6, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	/**
	 * Find controller for the statement after the 
	 * for loop in case of an unstructured code. Using 
	 * break statement to make the code unstructured.
	 */
	@Test
	public void forLoopWithPEIOffTest2(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.forLoopWithPEIOffTest2();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 9, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the invocation.", - 1, secondController.getSourceCodeLineNumber());
	}
	
	/**
	 * Find controller for the statement after the 
	 * break statment in case of an unstructured code. 
	 * Using break statement to make the code unstructured.
	 */
	@Test
	public void forLoopWithPEIOffTest3(){
		Analyzer.includePEIs = false;
		CodeSnippetCollection.forLoopWithPEIOffTest3();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the if dominating break.", slicingCriterionLineNo - 3, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the loop condition.", slicingCriterionLineNo - 5, secondController.getSourceCodeLineNumber());
	}


	
	/**
	 * Find controller for the statement after the 
	 * for loop
	 */
	@Test
	public void forLoopWithPEIOnTest1(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.forLoopWithPEIOnTest1();
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		System.out.println("ready to analyze");
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the loop condition.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the PEI.", slicingCriterionLineNo - 3, secondController.getSourceCodeLineNumber());
		
		BytecodeEvent thirdController = QueryFacade.findControllerForBytecode(secondController, session);
		assertEquals("Must be control dependent on the loop condition.", slicingCriterionLineNo - 4, thirdController.getSourceCodeLineNumber());

		BytecodeEvent forthController = QueryFacade.findControllerForBytecode(thirdController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 5, forthController.getSourceCodeLineNumber());

	}

	/**
	 * Find controller for the statement within the 
	 * for loop after the PEI
	 */
	@Test
	public void forLoopWithPEIOnTest2(){
		Analyzer.includePEIs = true;
		CodeSnippetCollection.forLoopWithPEIOnTest2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the if stmt dominating break.", slicingCriterionLineNo - 3, directController.getSourceCodeLineNumber());
		
		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the PEI.", slicingCriterionLineNo - 5, secondController.getSourceCodeLineNumber());
		
		BytecodeEvent thirdController = QueryFacade.findControllerForBytecode(secondController, session);
		assertEquals("Must be control dependent on the loop condition.", slicingCriterionLineNo - 6, thirdController.getSourceCodeLineNumber());

		BytecodeEvent forthController = QueryFacade.findControllerForBytecode(thirdController, session);
		assertEquals("Must be control dependent on the if stmt occuring in former iteration.", slicingCriterionLineNo - 3, forthController.getSourceCodeLineNumber());
	}
	
	
	
	
	/* ***********************************************************************************
	 * ***********************  Tests for short circuited condition*********************** 
	 *************************************************************************************/
	
	/**
	 * Find controller for the statement after the inner if.
	 * The inner if condition is not taken because of the 
	 * first condition.
	 */
	@Test
	public void shortCircuitAndOpWithoutPEITest1(){
		CodeSnippetCollection.shortCircuitAndOpWithoutPEITest1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 4, directController.getSourceCodeLineNumber());

	}

	
	/**
	 * Find controller for the statement within the inner if.
	 * The inner if condition IS taken because of the FIRST 
	 * condition.
	 */
	@Test
	public void shortCircuitAndOpWithoutPEITest2(){
		CodeSnippetCollection.shortCircuitAndOpWithoutPEITest2();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the inner if.", slicingCriterionLineNo - 1, directController.getSourceCodeLineNumber());

		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 2, secondController.getSourceCodeLineNumber());
	}

	
	/**
	 * Find controller for the statement within the inner if.
	 * The inner if condition IS taken because of the SECOND
	 * condition in the if statement.
	 */
	@Test
	public void shortCircuitAndOpWithoutPEITest3(){
		CodeSnippetCollection.shortCircuitAndOpWithoutPEITest3();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(2, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		
		final BytecodeEvent directController = QueryFacade.findControllerForBytecode(criterion, session);
		assertEquals("Must be control dependent on the second condition of inner if.", slicingCriterionLineNo - 1, directController.getSourceCodeLineNumber());

		BytecodeEvent secondController = QueryFacade.findControllerForBytecode(directController, session);
		assertEquals("Must be control dependent on the first condition of inner if.", slicingCriterionLineNo - 1, secondController.getSourceCodeLineNumber());
		
		BytecodeEvent thirdController = QueryFacade.findControllerForBytecode(secondController, session);
		assertEquals("Must be control dependent on the outer if.", slicingCriterionLineNo - 2, thirdController.getSourceCodeLineNumber());

	}

	
	/* ********************************************************************************
	 **********************************************************************************
	 **********************************************************************************/

	
	private void dumpTrace(RuntimeEnv session){
		List<BytecodeEvent> events = session.getAllBytecodeEvents();
		for (BytecodeEvent event : events) {
			System.out.println(event);
		}
	}

}



