package ch.usi.inf.sape.tracer.test;

import static org.junit.Assert.*;

import java.util.List;
import java.util.Set;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestWatcher;
import org.junit.runner.Description;

import ch.usi.inf.sape.tracer.analyzer.EventConstants;
import ch.usi.inf.sape.tracer.analyzer.QueryFacade;
import ch.usi.inf.sape.tracer.analyzer.bytecodeops.BytecodeEvent;
import ch.usi.inf.sape.tracer.analyzer.RuntimeEnv;
import ch.usi.inf.sape.tracer.analyzer.bytecodeops.SlicedBytecodeEvent;
import ch.usi.inf.sape.tracer.test.snippets.SlicingSnippetCollection;

/**
 * TODO Add tests for different combination of data 
 * and control dependencies.
 * 
 * TODO: Add tests for backward and forward slicing
 * 
 * @author reza
 */
public class SlicingTest {

	/**
	 * Logs each test method's name upon execution
	 */
	@Rule
	public TestWatcher testWatcher = new TestWatcher() {
		@Override
		protected void starting(final Description description) {
			String methodName = description.getMethodName();
			System.out.println("Test Method: " + methodName);
		}
	};

	
	/* ******************************************************************************************
	 ********************************************************************************************
	 ********************************************************************************************/

	/**
	 * Slicing criterion = last assignment stmt
	 */
	@Test
	public void flowThroughConditionalStmt(){
		SlicingSnippetCollection.flowThroughConditionalStmt();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		Set<Integer> backwardSlice = QueryFacade.findBackwardSliceForBytecodeAsSourceLineNo(criterion, true, true, session);
		
		assertTrue("The slicing criterion must be included.",backwardSlice.contains(slicingCriterionLineNo));
		assertTrue("Body of if condition must be included.",backwardSlice.contains(slicingCriterionLineNo - 2));		
		assertTrue("The if condition must be included.", backwardSlice.contains(slicingCriterionLineNo - 3));		
		assertTrue("Initialization of newValue must be included.", backwardSlice.contains(slicingCriterionLineNo - 4));		
		assertTrue("Initialization of cond must be included.", backwardSlice.contains(slicingCriterionLineNo - 5));	
		assertTrue("Initialization of temp must be included.", backwardSlice.contains(slicingCriterionLineNo - 6));
		
		assertFalse("Initial value of criterion must not be included.", backwardSlice.contains(slicingCriterionLineNo - 7));	
	}

	/**
	 * Slicing criterion = last assignment stmt
	 * The if condition is not taken
	 */
	@Test
	public void flowOffConditionalStmtTest1(){
		SlicingSnippetCollection.flowOffConditionalStmtTest1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		Set<Integer> backwardSlice = QueryFacade.findBackwardSliceForBytecodeAsSourceLineNo(criterion, true, true, session);
		
		assertTrue("The slicing criterion must be included.",backwardSlice.contains(slicingCriterionLineNo));
		assertTrue("The outer if must be included.", backwardSlice.contains(slicingCriterionLineNo - 5));	
		assertTrue("Initialization of cond must be included.", backwardSlice.contains(slicingCriterionLineNo - 6));
		assertTrue("Initialization of temp must be included.", backwardSlice.contains(slicingCriterionLineNo - 7));
		assertTrue("Initial value of criterion must be included.", backwardSlice.contains(slicingCriterionLineNo - 8));	
		
		assertFalse("Body of if condition must not be included.",backwardSlice.contains(slicingCriterionLineNo - 2));		
		assertFalse("The if condition must not be included.", backwardSlice.contains(slicingCriterionLineNo - 3));		
		assertFalse("Initialization of newValue must not be included.", backwardSlice.contains(slicingCriterionLineNo - 4));		
		
	}
	
	
	/**
	 * Slicing criterion = last assignment stmt
	 * The outer if condition is taken,
	 * The inner if condition is not taken.
	 */
	@Test
	public void flowDynamicTest1(){
		SlicingSnippetCollection.flowDynamicTest1(9);
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		Set<Integer> backwardSlice = QueryFacade.findBackwardSliceForBytecodeAsSourceLineNo(criterion, true, true, session);
		
		assertTrue("The slicing criterion must be included.",backwardSlice.contains(slicingCriterionLineNo));
		assertTrue("The outer if must be included.", backwardSlice.contains(slicingCriterionLineNo - 5));	
		assertTrue("Initialization of temp must be included.", backwardSlice.contains(slicingCriterionLineNo - 6));
		assertTrue("Initial value of criterion must be included.", backwardSlice.contains(slicingCriterionLineNo - 7));	
		
		assertFalse("Body of if condition must not be included.",backwardSlice.contains(slicingCriterionLineNo - 2));		
		assertFalse("The if condition must not be included.", backwardSlice.contains(slicingCriterionLineNo - 3));		
		assertFalse("Initialization of newValue must not be included.", backwardSlice.contains(slicingCriterionLineNo - 4));		
		
	}

	/**
	 * Slicing criterion = last assignment stmt
	 * The outer if condition is taken,
	 * The while is repeated couple of times.
	 */
	@Test
	public void flowDynamicTest2(){
		SlicingSnippetCollection.flowDynamicTest2(20);
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		Set<Integer> backwardSlice = QueryFacade.findBackwardSliceForBytecodeAsSourceLineNo(criterion, true, true, session);
		
		assertTrue("The slicing criterion must be included.",backwardSlice.contains(slicingCriterionLineNo));
		assertTrue("The outer if must be included.", backwardSlice.contains(slicingCriterionLineNo - 5));	
		assertTrue("Initialization of temp must be included.", backwardSlice.contains(slicingCriterionLineNo - 6));
		assertTrue("Initial value of criterion must be included.", backwardSlice.contains(slicingCriterionLineNo - 7));	
		
		assertFalse("Body of if condition must not be included.",backwardSlice.contains(slicingCriterionLineNo - 2));		
		assertFalse("The if condition must not be included.", backwardSlice.contains(slicingCriterionLineNo - 3));		
		assertFalse("Initialization of newValue must not be included.", backwardSlice.contains(slicingCriterionLineNo - 4));		
	}
	
	
	/**
	 * Slicing criterion = last assignment stmt
	 * 
	 */
	@Test
	public void longExecutionTest1(){
		SlicingSnippetCollection.longExecutionTest1();
		
		RuntimeEnv session = RuntimeEnv.getInstance();
		
		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, session);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		Set<Integer> backwardSlice = QueryFacade.findBackwardSliceForBytecodeAsSourceLineNo(criterion, true, true, session);
		
		assertTrue("The slice must include the initialization of the element.",backwardSlice.contains(slicingCriterionLineNo - 11));
		assertTrue("The slice must include the initialization of array reference.",backwardSlice.contains(slicingCriterionLineNo - 13));
		assertTrue("The slice must include the swapping operation.",backwardSlice.contains(slicingCriterionLineNo +12));
		assertTrue("The slice must include the condition variable for while loop.",backwardSlice.contains(slicingCriterionLineNo + 16));
		assertTrue("The slice must include the swapping operation.",backwardSlice.contains(slicingCriterionLineNo + 30));
		assertTrue("The slice must include the swapping operation.",backwardSlice.contains(slicingCriterionLineNo + 31));
		
		assertFalse("The slice must not include the overwritten initialization of the elements.",backwardSlice.contains(slicingCriterionLineNo - 12));
		assertFalse("The slice must not include the dummy initialization.",backwardSlice.contains(slicingCriterionLineNo + 9));
		assertFalse("The slice must not include the dummy initialization.",backwardSlice.contains(slicingCriterionLineNo + 11));
		assertFalse("The slice must not include the dummy initialization.",backwardSlice.contains(slicingCriterionLineNo + 15));
		assertFalse("The slice must not include the dummy initialization.",backwardSlice.contains(slicingCriterionLineNo + 29));
		assertFalse("The slice must not include the dummy initialization.",backwardSlice.contains(slicingCriterionLineNo + 32));
		assertFalse("The slice must not include the dummy initialization.",backwardSlice.contains(slicingCriterionLineNo + 33));
	}

	@Test
	public void pathTest(){
		SlicingSnippetCollection.pathTest();
		
		RuntimeEnv env = RuntimeEnv.getInstance();

		BytecodeEvent criterion = QueryFacade.findNthLastOccurrenceOfEventClass(1, EventConstants.XSTORE_EVENT, env);
		final int slicingCriterionLineNo = criterion.getSourceCodeLineNumber();
		Set<Integer> backwardSlice = QueryFacade.findBackwardSliceForBytecodeAsSourceLineNo(criterion, true, true, env);
		
		assertTrue("The slice must include the condition variable inside isEmpty method.",backwardSlice.contains(slicingCriterionLineNo + 8));
	}
	
}
