package ch.usi.inf.sape.tracer.test.snippets;

import java.io.IOException;

/**
 * This class contains some code snippets to be
 * used by the unit tests as the sample scenarios.
 * 
 * For every junit test, we define the scenario
 * here as a static method and invoke it from
 * within the junit test method. This is to 
 * avoid the instrumentation of junit test methods.
 * The instrumentation happens as soon as a method 
 * of this class is invoked.
 * 
 * As for the backward slicing test,
 * the predicates are sensitive to the source line
 * numbers. So, as soon as we are done with writing
 * a scenario, we are not allowed to reorder the lines
 * or add a line break in between. 
 * 
 * @author reza
 *
 */
public class CodeSnippetCollection {

	
	/* ****************************************************************
	 ********************** IF_X Operations ***************************
	 ******************************************************************/

	/**
	 * Pre: only one if condition allowed
	 */
	public static int ifopWithoutPEI1(){
		int a = 12;
		long b = 23;
		if (b > 0){
			if (a > 0){
				//Do not remove the break line
				b = 4;
			}
			b = b + a;
		}
		return a;
	}
	
	public static int ifopWithoutPEI2(){
		int a = 12;
		long b = 23;
		if (a > 0){
			if (b < a){
				b = 4;
			}
			//Do not remove the break line
			b = b + a;
		}
		return a;
	}
	
	public static void ifopWithPEI1(){
		Object obj = new Object();
		int a = 12;
		long b = 23;
		if (b == 23){
			if (a > 0){
				b = obj.hashCode();
			}
			//Do not remove the break line
			b = b + a;
		}
	}

	public static void ifopWithPEI2(){
		int[] array = new int[10];
		int a = 12;
		long b = 23;
		if (b == 23){
			if (a > 0){
				b = array.length;
			}
			//Do not remove the break line
			b = b + a;
		}
	}
	
	public static void ifopWithPEI3(){
		Object obj = new Object();
		int a = 12;
		long b = 23;
		if (b == 23){
			if (a < 0){
				b = obj.hashCode();
			}
			//Do not remove the break line
			b = b + a;
		}
	}
	
	public static boolean ifopWithPEI4(){
		Object obj = new Object();
		int a = 12;
		long b = 23;
		if (b == 23){
			if (a < 0){
				b = 78;
				return true;
			}
			//Do not remove the break line
			b = b + a;
			return false;
		}
		return true;
	}


	/* ****************************************************************
	 ****************** IF_ICMPX Operations ***************************
	 ******************************************************************/
	
	public static int ificmpXopWithoutPEI1(){
		int a = 12;
		int b = 23;
		if (a < 50){
			if (b > a){
				b = 4;
			}
			b = b + a;
		}
		return a;
	}
	
	public static int ificmpXopWithoutPEI2(){
		int a = 12;
		int b = 23;
		if (a < 50){
			if (b < a){
				b = 4;
			}
			b = b + a;
		}
		return a;
	}
	
	public static void ificmpXopWithPEI1(){
		Object obj = new Object();
		int a = 12;
		int b = 23;
		if (a < b){
			if (a < 14){
				b = obj.hashCode();
			}
			b = b + a;
		}
	}

	public static void ificmpXopWithPEI2(){
		int[] array = new int[10];
		int a = 12;
		int b = 23;
		if (a < b){
			if (a > 6){
				b = array.length;
			}
			b = b + a;
		}
	}
	
	public static void ificmpXopWithPEI3(){
		Object obj = new Object();
		int a = 12;
		int b = 23;
		if (b == 23){
			if (a == 11){
				b = obj.hashCode();
			}
			b = b + a;
		}
	}
	
	
	/* ****************************************************************
	 ****************** IF_ACMPX Operations ***************************
	 ******************************************************************/
	
	public static int ifacmpXopWithoutPEI1(){
		Object o1 = new Object();
		Object o2 = new Object();
		Object o3 = o1;
		int b = 1;
		
		if (o1 != o2){
			if (o1 == o3){
				b = 4;
			}
			b = 3;
		}
		return b;
	}
	
	public static int ifacmpXopWithoutPEI2(){
		Object o1 = new Object();
		Object o2 = new Object();
		Object o3 = o1;
		int b = 1;
		
		if (o2 != o3){
			if (o1 == o3){
				b = 4;
			}
			b = 5;
		}
		return b;
	}
	
	public static void ifacmpXopWithPEI1(){
		Object o1 = new Object();
		Object o2 = new Object();
		Object o3 = o1;
		int b = 1;
		
		if (o2 != o3){
			if (o1 == o3){
				b = o1.hashCode();
			}
			b = 5;
		}
	}

	public static void ifacmpXopWithPEI2(){
		Object o1 = new Object();
		Object o2 = new Object();
		Object o3 = o1;
		int b = 1;
		int[] array = new int[10];
		
		if (o1 == o3){
			if (o2 != o3){
				b = array.length;
			}
			b = 4;
		}
	}
	
	public static void ifacmpXopWithPEI3(){
		Object o1 = new Object();
		Object o2 = new Object();
		Object o3 = o1;
		int b = 1;
		
		if (o1 == o3){
			if (o2 == o3){
				b = o1.hashCode();
			}
			b = 6;
		}
	}

	
	/* ****************************************************************
	 ****************** tableswitch and lookupswitch ******************
	 ******************************************************************/
	
	public static void lookupSwitchWithoutPEI1(){
		String cond = "Hi";
		int dummy = 5;
		if (dummy > 0){
			switch(cond){
			case "Ho":
				dummy = 1;
				break;
			case "Hi":
				dummy = 2;
				break;
			case "Ha":
				dummy = 3;
				break;
			default:
				dummy = 6;
				break;
			}
			dummy = 7;
		}
	}

	public static void tableSwitchWithoutPEI1(){
		int cond = 5;
		int dummy = 5;
		if (dummy > 0){
			switch(cond){
			case 1:
				dummy = 1;
				break;
			case 3:
				dummy = 2;
				break;
			case 5:
				dummy = 3;
				break;
			default:
				dummy = 6;
				break;
			}
			dummy = 7;
		}
	}
	
	public static void tableSwitchWithPEI1(){
		int dummy = 8;
		if (dummy > 0){
			switch(dummy){
			case 1:
				dummy = tableswitchUtility(1);
				break;
			case 2:
				dummy = tableswitchUtility(2);
				break;
			case 5:
				dummy = tableswitchUtility(3);
				break;
			default:
				dummy = 4;
				break;
			}
			dummy = 7;
		}
	}
	
	public static void lookupSwitchWithPEI1(){
		char cond = 3;
		int dummy = 5;
		if (dummy > 0){
			switch(tableswitchUtility(3)){
			case 'a':
				dummy = 1;
				break;
			case 'Z':
				dummy = 2;
				break;
			case '5':
				dummy = 3;
				break;
			default:
				dummy = 6;
				break;
			}
		}
		dummy = 7;
	}
	
	public static void lookupSwitchWithPEI2(){
		String cond = "Hi";
		int dummy = 5;
		if (dummy > 0){
			switch(cond){
			case "Ho":
				dummy = tableswitchUtility(1);
				break;
			case "Hi":
				dummy = tableswitchUtility(2);
				break;
			case "Ha":
				dummy = tableswitchUtility(3);
				break;
			default:
				dummy = tableswitchUtility(4);
				break;
			}
			dummy = 7;
		}
	}


	private static int tableswitchUtility(int arg){
		int ret;
		switch(arg){
		case 1:
			ret = 1;
			break;
		case 2:
			ret = 2;
			break;
		case 3:
			ret = 3;
			break;
		default:
			ret = 4;
			break;
		}
		
		return ret;
	}
	
	
	/* ********************************************************************************
	 *************************  Loops and goto bytecode  ******************************
	 **********************************************************************************/
	
	public static void forLoopWithoutPEI(){
		int iteration = 2;
		int count = 0;
		if (count >= 0){
			for (int i = 0; i < iteration; i++) {
				count++;
			}
			count = 10;
		}
	}
	
	public static void forLoopWithoutPEI2(){
		int iteration = 3;
		int count = 0;
		if (count >= 0){
			for (int i = 0; i < iteration; i++) {
				count = i;
			}
			count = 10;
		}
	}
	
	public static void forLoopWithoutPEITest3(){
		int iteration = 4;
		int count = 0;
		if (count >= 0){
			for (int i = 0; i < iteration; i++) {
				//don't remove this line
				if (i == 2)
					break;
				//don't remove this line
				count = i;
			}
			count = 10;
		}
	}


	public static void forLoopWithPEI1(){
		Object obj = new Object();
		int iteration = 2;
		int count = 0;
		if (count >= 0){
			for (int i = 0; i < iteration; i++) {
				obj.hashCode();
				count++;
			}
			//don't remove this line
			count = 10;
		}
	}
	
	public static void forLoopWithPEIOffTest2(){
		Object obj = new Object();
		int iteration = 4;
		int count = 0;
		if (count >= 0){
			for (int i = 0; i < iteration; i++) {
				obj.hashCode();
				if (count == 2)
					break;
				//don't remove this line
				count++;
			}
			//don't remove this line
			count = 10;
		}
	}

	public static void forLoopWithPEIOffTest3(){
		Object obj = new Object();
		int iteration = 4;
		int count = 0;
		if (count >= 0){
			for (int i = 0; i < iteration; i++) {
				obj.hashCode();
				if (count == 2)
					break;
				//don't remove this line
				count = i;
			}
			//don't remove this line
			count = 10;
		}
	}

	
	public static void forLoopWithPEIOnTest1(){
		Object obj = new Object();
		int iteration = 1;
		int count = 0;
		if (count >= 0){
			for (int i = 0; i < iteration; i++) {
				obj.hashCode();
				count++;
			}
			count = 10;
		}
	}

	public static void forLoopWithPEIOnTest2(){
		Object obj = new Object();
		int iteration = 4;
		int count = 0;
		if (count >= 0){
			for (int i = 0; i < iteration; i++) {
				obj.hashCode();
				//don't remove this line
				if (i == 2)
					break;
				//don't remove this line
				count = i;
			}
			count = 10;
		}
	}


	
	
	/* ********************************************************************************
	 ************************Tests for short circuited condition***********************
	 **********************************************************************************/
	
	public static void shortCircuitAndOpWithoutPEITest1(){
		int dummy1 = 5;
		int dummy2 = 12;
		if (dummy1 > 0){
			if (dummy1 >= 7 && dummy2 < 15){
				dummy1 = 0;
			}
			dummy1 = 4;
		}
	}
	
	public static void shortCircuitAndOpWithoutPEITest2(){
		int dummy1 = 5;
		int dummy2 = 12;
		if (dummy1 > 0){
			if (dummy1 < 7 || dummy2 < 15){
				dummy1 = 0;
			}
			dummy1 = 4;
		}
	}

	public static void shortCircuitAndOpWithoutPEITest3(){
		int dummy1 = 5;
		int dummy2 = 12;
		if (dummy1 > 0){
			if (dummy1 > 7 || dummy2 < 15){
				dummy1 = 0;
			}
			dummy1 = 4;
		}
	}

	/* ********************************************************************************
	 **********************************************************************************
	 **********************************************************************************/
	
	public static void invocationStatic(){
		int a = 5;
		if (a <= 10)
			a = ifacmpXopWithoutPEI1();
	}

	public static void invocationNonStatic(){
		CodeSnippetCollection snippet = new CodeSnippetCollection();
		int a = 5;
		if (a <= 10)
			a = snippet.nonStaticMethod(12);
	}
	
	private int nonStaticMethod(int arg){
		int var = 12;
		if (var == arg){
			return 3;
		}
		
		return 5;
	}

	public static void catchRuntimeException(){
		int var = -12;
		try{
			int[] array = new int[var];
		}catch(RuntimeException e){
			var = 13;
		}
	}
	
	public static void catchRuntimeExceptionAfterInvocation(){
		int var = -12;
		try{
			if (var < 0){
				throwRuntimeException(var);
			}
		}catch(RuntimeException e){
			var = 13;
		}
	}
	/* Don't change this gap between the preceeding and the following method,
	 because the analysis is sensitive to the line numbers */
	private static void throwRuntimeException(int arg){
		int[] array = new int[arg];
	}
	
	public static void catchCheckedException(){
		boolean arg = true;
		try{
			if (arg){
				throwCheckedException(arg);
			}
		}catch(IOException e){
			arg = false;
		}
	}
	/* Don't change this gap between the preceeding and the following method,
	 because the analysis is sensitive to the line numbers */
	private static void throwCheckedException(boolean arg) throws IOException{
		if (arg)
			throw new IOException();
	}
	
}
